/*
 * SPDX-FileCopyrightText: 2019 Bruce Cowan <bruce@bcowan.me.uk>
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include "slist.h"

#include <stdlib.h>
#include <string.h>

SList *
slist_prepend (SList *list,
               void  *data)
{
  SList *new = malloc (sizeof (SList));
  new->data = data;
  new->next = list;

  return new;
}

SList *
slist_find (SList      *list,
            const void *data)
{
  for (SList *l = list; l; l = l->next)
    {
      if (l->data == data)
        return l;
    }

  return NULL;
}

SList *
slist_nth (SList *list,
           int    idx)
{
  int i = 0;

  for (SList *l = list; l; l = l->next)
    {
      if (i == idx)
        return l;
      i++;
    }

  return NULL;
}

void
slist_foreach (SList      *list,
               ForEachFunc func,
               void       *user_data)
{
  for (SList *l = list; l; l = l->next)
    func (l->data, user_data);
}

static inline void
slist_free_one (SList   *list,
                FreeFunc func)
{
  if (func)
    func (list->data);

  free (list);
}

SList *
slist_remove (SList   *list,
              void    *data,
              FreeFunc func)
{
  if (!list)
    return NULL;

  // If the first element is the one we're looking for
  if (list->data == data)
    {
      SList *head = list->next;
      slist_free_one (list, func);
      return head;
    }

  SList *prev = list;
  for (SList *l = list; l; prev = l, l = l->next)
    {
      if (l->data == data)
        {
          prev->next = l->next;
          slist_free_one (l, func);
          break;
        }
    }

  return list;
}

void
slist_free_all (SList    *list,
                FreeFunc  func)
{
  slist_foreach (list, (ForEachFunc) slist_free_one, NULL);
}
